<?	/*
	// File:	config.inc.php
	// Purpose:	web://cp interface configurations defaults
	// Author:	Felix <webmaster@can-host.com>
	*/
/////////////////////////////////////////////////////////
// Default Configurations
//
// Instructions:
// 1. DO NOT EDIT THIS FILE
// 2. Create a file called config.php in webcp's base directory.
// 3. Copy the $cfg values you need to change for your system.
//	  Only copy those values that differ from their default value.  
// 4. Please read INSTALL for detailed instructions.
//
// Tips:
// 1. If you NEED to use a ' symbol, you MUST write it like this: \'
// 2. For config settings that do not apply do  one of the following:
//	 1. Comment out the setting (eg. // $cfg['setting'] = 'setting';)
//	 2. Set the setting equal to an empty string (eg. $cfg['setting'] = '';)
//
// Table Of Contents:
// 0. Installation Settings
// 1. Control Panel Settings
// 2. CRON Settings
// 3. HTTP Server Config
// 4. HTTP Server Config (Control Panel)
// 5. Database Server Config (Control Panel)
// 6. System Config
// 7. Operating System
// 8. Hardware
// 9. Program Locations
// 10. Server Selection
// 11. Server Config
// 12. Services (config & monitoring)
// 13. DNS Config
// 14. Mail Exchangers
// 15. Helpdesk Settings
/////////////////////////////////////////////////////////
// 0. Installation Settings
//
$cfg['programslocated'] = false;
$cfg['serversselected'] = false;
/////////////////////////////////////////////////////////
// 1. Control Panel Settings
//
// CP Administrator E-Mail
$cfg['adminmail']		= '';
// CP Web Name
if (isset($_SERVER['SERVER_NAME'])) {
	$cfg['sysname']		=  $_SERVER['SERVER_NAME'];
} else {
	$cfg['sysname'] = 'localhost';
}
// CP Web Name
$cfg['ftpsysname']	= 'localhost';
$cfg['ftpsysport']	= '21';
// root cp server. (localhost|ip_addr|none)
$cfg['root_server']	= '';
// dns://cp server. (localhost|ip_addr|none)
$cfg['dns_server']	= '';
// Base Directory (/home/admin)
$cfg['basedir']		= '/home/webcp';
// Base Web Directory (/home)
$cfg['webdir']		= '/home';
// Web Documents Directory Name (web)
$cfg['webname']		= 'web';
// Cookie security.  Set to false to use web://cp with 'unsafe' cookies.  Set to true when the DNS is set.
$cfg['cookiesec']		= true;
// Log Level: 0 for none, 1 for fatal errors, 2 for +warning, 3 for +activity logging
$cfg['loglevel']		= 2;
// Which log system to use (file|db|filedb|email)
// filedb = file + db
$cfg['log_system']      = 'db';
// Where do you want to save log file
// don't forget to chown
$cfg['logfile']         = '/var/log/webcp';
// Apache central access log file for logging cp access
$cfg['accesslog']		= '/var/log/httpd/access';
// Max allowed updates per domain & user (from 0 to 99)
$cfg['ucount']		= 25;
// Limit Bandwidth on client sites via mod_bandwidth (true|false)
$cfg['bandwidth']		= false;
// Server Side Languages (php,asp,perl,ssi,etc)
$cfg['sslang']		= 'php,ssi,perl';
// Supported Languages (lang1,lang2,lang3)
$cfg['lang']		= 'english,french,spanish';
// Default Language (edit carefully)
$cfg['defaultlang']	= 'english';
// Default web interface skin
$cfg['defaultskin']	= 'greenclassic';
// secret crypt key for encryption purpose
$cfg['key']			= 'SECRET_KEY';
// web://cp version
$cfg['webcp']		= '0.5.7';
// Configuration options we allow to be changed or added from the front end.
// These are the variables requested by the setup.php page
// Do not change unless you know what you're doing
$cfg['allowed_cfg'] 	= 'dbhost|dbuser|dbpass|dbname|sysname|adminmail|os|osversion|defaultlang|key|httpd_mode|programslocated|serversselected|httpserver|mailserver|maillist|ftpserver|dns_system|dns2_system|log_system';
// web://cp Daemon PID save location
$cfg['pid']			= '/var/run/webcp/webcp.pid';
// Invalid users (i.e not in web.cp but elsewhere in the system).  don't trim() spaces
$cfg['badusers']		= ' root bin daemon adm lp sync shutdown halt mail news uucp operator games gopher ftp nobody nscd mailnull ident rpc xfs httpd apache named cp webcp mysql template1 ';
// use strong-password check to ensure passwords are not weak?
$cfg['strong_passwords']= true;
$cfg['cracklib_dict'] = '/usr/local/lib/pw_dict';
// number of files allowed to be uploaded at 1 time
$cfg['upload_files_num'] = 5;
// users are chrooted when using FTP (tells file manager whether to allow symlinks)
$cfg['users_chrooted'] = true;
// should non-server users be forbidden from logging in during backend outages
$cfg['preventoutagelogins'] = true;
$cfg['statustag'] = "/tag/.status";
// Temp directory
$cfg['tmpdir'] = '/tmp';
/////////////////////////////////////////////////////////
// 2. CRON Settings
//
// Time for hourly
$cfg['cron_hourly']['minute'] 	= 1;
// Time for multi-daily
$cfg['cron_multidaily']['minute'] 	= 30;
$cfg['cron_multidaily']['hour'] 	= array(3, 15);
// Time for daily
$cfg['cron_daily']['minute'] 		= 30;
$cfg['cron_daily']['hour'] 		= 3;
// Time for weekly
$cfg['cron_weekly']['minute'] 	= 30;
$cfg['cron_weekly']['hour'] 		= 7;
$cfg['cron_weekly']['weekday'] 	= 0;
// Time for monthly
$cfg['cron_monthly']['minute'] 	= 30;
$cfg['cron_monthly']['hour'] 		= 1;
$cfg['cron_monthly']['day'] 		= 1;
/////////////////////////////////////////////////////////
// 3. HTTP Server Configuration 
//
$cfg['apache']['port'] = "80";
/////////////////////////////////////////////////////////
// 4. HTTP Server Configuration (Control Panel)
//
// SSL support on httpd-cp (true|false)
$cfg['ssl']				= false;
// httpd-cp listening port (of the dedicated web://cp HTTPD server)
$cfg['port']			= 81;
// apache compatibilty or the webcp built in server (apache|webcp)
$cfg['httpd_mode'] 		= 'apache';
// internal port stunnel connects to if ssl is enabled for webcp-httpd
$cfg['ssl_port'] 			= '83';
// using webcp-httpd should we check php pages for incompatible code (true|false)
$cfg['validate_php'] 		= true;
// SSL certificate to use with stunnel. use:
// openssl req -new -x509 -nodes -out ssl-cert.pem -keyout ssl-cert.pem -days 365
$cfg['ssl_cert'] 			= $cfg['basedir'].'/httpd/conf/ssl-cert.pem';
// location of webcp-httpd pid file
$cfg['httpd_pid'] 		= '/var/run/webcp/webcp-httpd.pid';
// location of stunnel pid file
$cfg['httpsd_pid'] 		= '/var/run/webcp/webcp-stunel.pid';
// user that webcp-httpd will change to after running
$cfg['httpd_user'] 		= 'webcp';
// 0.0.0.0 will bind to all local addresses (could also use gethostbyname($cfg['sysname']))
$cfg['listen_address'] 		= '0.0.0.0';
// The maximum number of backlogged incoming connections to be queued for processing
$cfg['listen_queue'] 		= '20';
// The maximum number of client connections at once
$cfg['max_clients'] 		= '20';
// maximum amount of time in seconds to remain connected to a client.
$cfg['timeout'] 			= '100';
// allow persistent connections? (true|false)
$cfg['keep_alive'] 		= true;
// keep alive sockets are closed after this many seconds of inactivity
$cfg['keep_alive_timeout'] 	= '15';
/////////////////////////////////////////////////////////
// 5. Database Server Configurations
//
// web://cp uses MySQL as its backend, enter the root MySQL access info here.
// MySQL hostname (usually localhost)
// Remote database is not recommeded due to security implications
$cfg['dbhost']		= 'localhost';
// MySQL DB Username
$cfg['dbuser']		= 'webcp';
// MySQL DB password
$cfg['dbpass']		= 'PASSWORD';
// MySQL DB name
$cfg['dbname']		= 'webcp';
// Admin user (used in backups)
$cfg['dbadminuser']	= 'root';
// Admin password (used in backups)
$cfg['dbadminpass']	= '';
// PostgreSQL configuration.  web://cp can manage it, enter root access info here.
// PostgreSQL Database Host. (localhost|ip_addr|none)
$cfg['pgsql_host']	= '';
// PostgreSQL superuser username.
$cfg['pgsql_user']	= '';
// PostgreSQL superuser's password
$cfg['pgsql_pass']	= 'PASSWORD';
// Default PostgreSQL DB to connect to. (template1)
$cfg['pgsql_db']		= 'template1';
/////////////////////////////////////////////////////////
// 6. System Configuration
// 
/////////////////////////////////////////////////////////
// 7. OS Configurations
//
// OS Type (For OS specific tweaks)
// (linux|freebsd)
$cfg['os']			= 'linux';
// OS Version (and distribution)
// -- linux --
// RedHat6.2
// RedHat7.0 - 7.3
// Redhat9.0
// -- freebsd --
// 4
$cfg['osversion']		= 'RedHat9.0';
/////////////////////////////////////////////////////////
// 8. Hardware Configuration
//
// Hard Drive partition holding /home (for HD quotas management)
$cfg['devhd']		= '/dev/hda1';
// Network interface to manage IP on
$cfg['eth']			= 'eth0';
/////////////////////////////////////////////////////////
// 9. Program Locations
//
// PHP libs Location
$cfg['php_lib'] 		= '/usr/lib/php';
// Utility Program Location
$cfg['prog']['find']	= '/usr/bin/find';
$cfg['prog-req']['find'] = false;
$cfg['prog-desc']['find'] = "find is used to list files in a directory.";
$cfg['prog']['quota']	= '/usr/bin/quota';
$cfg['prog-req']['quota'] = false;
$cfg['prog-desc']['quota'] = "quota is needed in order to enforce HD usage quotas.";
$cfg['prog']['free']	= '/usr/bin/free';
$cfg['prog-req']['free'] = false;
$cfg['prog-desc']['free'] = "free is used to display memory usage.";
$cfg['prog']['uptime']	= '/usr/bin/uptime';
$cfg['prog-req']['uptime'] = false;
$cfg['prog-desc']['uptime'] = "uptime is used to display server uptime.";
$cfg['prog']['shut']	= '/sbin/shutdown';
$cfg['prog-req']['shut'] = false;
$cfg['prog-desc']['shut'] = "shut is used to shutdown the server.";
$cfg['prog']['makemap']	= '/usr/bin/makemap';
$cfg['prog-req']['makemap'] = false;
$cfg['prog-desc']['makemap'] = "makemap is used by sendmail.";
$cfg['prog']['nalias']	= '/usr/bin/newaliases';
$cfg['prog-req']['nalias'] = false;
$cfg['prog-desc']['nalias'] = "newaliases is used by sendmail.";
$cfg['prog']['gadd']	= '/usr/sbin/groupadd';
$cfg['prog-req']['gadd'] = true;
$cfg['prog-desc']['gadd'] = "gadd is used to add Unix user groups.";
$cfg['prog']['gdel']	= '/usr/sbin/groupdel';
$cfg['prog-req']['gdel'] = true;
$cfg['prog-desc']['gdel'] = "gdel is used to delete Unix user groups.";
$cfg['prog']['gmod']	= '/usr/sbin/groupmod';
$cfg['prog-req']['gmod'] = true;
$cfg['prog-desc']['gmod'] = "gmod is used to modify Unix user groups.";
$cfg['prog']['getent']	= '/usr/bin/getent';
$cfg['prog-req']['getent'] = false;
$cfg['prog-desc']['getent'] = "getent is used to determine if a Unix user exists.";
$cfg['prog']['uadd']	= '/usr/sbin/useradd';
$cfg['prog-req']['uadd'] = true;
$cfg['prog-desc']['uadd'] = "uadd is used to add Unix users.";
$cfg['prog']['udel']	= '/usr/sbin/userdel';
$cfg['prog-req']['udel'] = true;
$cfg['prog-desc']['udel'] = "udel is used to delete Unix users.";
$cfg['prog']['umod']	= '/usr/sbin/usermod';
$cfg['prog-req']['umod'] = true;
$cfg['prog-desc']['umod'] = "umod is used to modify Unix users.";
$cfg['prog']['squota']	= '/usr/sbin/setquota';
$cfg['prog-req']['squota'] = false;
$cfg['prog-desc']['squota'] = "squota is used to set quota limits.";
$cfg['prog']['cp']	= '/bin/cp';
$cfg['prog-req']['cp'] = true;
$cfg['prog-desc']['cp'] = "cp is used to copy files.";
$cfg['prog']['rm']	= '/bin/rm';
$cfg['prog-req']['rm'] = true;
$cfg['prog-desc']['rm'] = "rm is used to delete files.";
$cfg['prog']['sed']	= '/bin/sed';
$cfg['prog-req']['sed'] = false;
$cfg['prog-desc']['sed'] = "sed is used to read files.";
$cfg['prog']['chown']	= '/bin/chown';
$cfg['prog-req']['chown'] = true;
$cfg['prog-desc']['chown'] = "chown is used to change ownership of files.";
$cfg['prog']['shell']	= '/bin/bash';
$cfg['prog-req']['shell'] = false;
$cfg['prog-desc']['shell'] = "shell is a command line interface.";
$cfg['prog']['noshell']	= '/bin/false';
$cfg['prog-req']['noshell'] = true;
$cfg['prog-desc']['noshell'] = "noshell is a command line intergace substitute.";
$cfg['prog']['make']	= '/usr/bin/make';
$cfg['prog-req']['make'] = false;
$cfg['prog-desc']['make'] = "make is used to automatically compile programs.";
$cfg['prog']['stunnel']	= '/usr/sbin/stunnel';
$cfg['prog-req']['stunnel'] = false;
$cfg['prog-desc']['stunnel'] = "stunnel is used as an encryption wrapper.";
$cfg['prog']['mysql']	= '/usr/bin/mysql';
$cfg['prog-req']['mysql'] = true;
$cfg['prog-desc']['mysql'] = "mysql is used to administer the MySQL database.";
$cfg['prog']['mm_new']	= '/home/mailman/bin/newlist';
$cfg['prog-req']['mm_new'] = false;
$cfg['prog-desc']['mm_new'] = "mm_new is used to delete files.";
$cfg['prog']['mm_rm']	= '/home/mailman/bin/rmlist -a';
$cfg['prog-req']['mm_rm'] = false;
$cfg['prog-desc']['mm_rm'] = "mm_rm is used to delete mailman mailing lists.";
// FreeBSD programs
$cfg['prog']['pw']	= '/usr/sbin/pw';
$cfg['prog-req']['pw'] = false;
$cfg['prog-desc']['pw'] = "pw is used to.";
// Webalizer Location
$cfg['prog']['walizer'] = '/usr/bin/webalizer';
$cfg['prog-req']['walizer'] = false;
$cfg['prog-desc']['walizer'] = "mm_rm is used to delete mailman mailing lists.";
// Pure-FTPD-pw
$cfg['prog']['pureftpd'] 	= '/usr/local/bin/pure-pw';
$cfg['prog-req']['pureftpd'] = false;
$cfg['prog-desc']['pureftpd'] = "mm_rm is used to delete mailman mailing lists.";
// qmail/vpopmail
$cfg['vqmail_home']			= '/home/vpopmail';
$cfg['prog']['autorespond']		= '/usr/local/bin/autorespond';
$cfg['prog-req']['autorespond'] = false;
$cfg['prog-desc']['autorespond'] = "autorespond is used to.";
$cfg['prog']['vadddomain']		= $cfg['vqmail_home'].'/bin/vadddomain';
$cfg['prog-req']['vadddomain'] = false;
$cfg['prog-desc']['vadddomain'] = "vadddomain is used to.";
$cfg['prog']['vdeldomain']		= $cfg['vqmail_home'].'/bin/vdeldomain';
$cfg['prog-req']['vdeldomain'] = false;
$cfg['prog-desc']['vdeldomain'] = "vdeldomain is used to.";
$cfg['prog']['vadduser']		= $cfg['vqmail_home'].'/bin/vadduser';
$cfg['prog-req']['vadduser'] = false;
$cfg['prog-desc']['vadduser'] = "vadduser is used to.";
$cfg['prog']['vdeluser']		= $cfg['vqmail_home'].'/bin/vdeluser';
$cfg['prog-req']['vdeluser'] = false;
$cfg['prog-desc']['vdeluser'] = "vdeluser is used to.";
$cfg['prog']['vpasswd']			= $cfg['vqmail_home'].'/bin/vpasswd';
$cfg['prog-req']['vpasswd'] = false;
$cfg['prog-desc']['vpasswd'] = "vpasswd is used to.";
$cfg['prog']['vmoduser']		= $cfg['vqmail_home'].'/bin/vmoduser';
$cfg['prog-req']['vmoduser'] = false;
$cfg['prog-desc']['vmoduser'] = "vmoduser is used to.";
$cfg['prog']['vaddaliasdomain']	= $cfg['vqmail_home'].'/bin/vaddaliasdomain';
$cfg['prog-req']['vaddaliasdomain'] = false;
$cfg['prog-desc']['vaddaliasdomain'] = "vaddaliasdomain is used to.";
$cfg['prog']['vdelivermail']		= $cfg['vqmail_home'].'/bin/vdelivermail';
$cfg['prog-req']['vdelivermail'] = false;
$cfg['prog-desc']['vdelivermail'] = "vdelivermail is used to.";
// Network Utilities
$cfg['prog']['ping']		= '/bin/ping';
$cfg['prog-req']['ping'] = false;
$cfg['prog-desc']['ping'] = "ping is used to.";
$cfg['prog']['traceroute']		= '/bin/traceroute';
$cfg['prog-req']['traceroute'] = false;
$cfg['prog-desc']['traceroute'] = "traceroute is used to.";
$cfg['prog']['whois']		= '/bin/whois';
$cfg['prog-req']['whois'] = false;
$cfg['prog-desc']['whois'] = "whois is used to.";
$cfg['prog']['dig']		= '/bin/dig';
$cfg['prog-req']['dig'] = false;
$cfg['prog-desc']['dig'] = "dig is used to.";
// authconfig (for crypt or md5 passwords)
$cfg['authconfig'] 	= '/etc/sysconfig/authconfig';
// MySQL
$cfg['prog']['mysqlcheck']		= '/usr/bin/mysqlcheck';
$cfg['prog-req']['mysqlcheck'] = false;
$cfg['prog-desc']['mysqlcheck'] = "mysqlcheck is used to check the integrity of and repair MySQL databases.";
$cfg['prog']['mysqldump']		= '/usr/bin/mysqldump';
$cfg['prog-req']['mysqldump'] = false;
$cfg['prog-desc']['mysqldump'] = "mysqldump is used to backup MySQL databases.";
/////////////////////////////////////////////////////////
// 10. Server Selection
//
// HTTP Server (apache1|apache2)
$cfg['httpserver']		= 'apache2';
// FTP Server (none|wuftpd|proftpd|pureftpd)
$cfg['ftpserver']		= 'proftpd';
// Mailing List support (none|mailman)
$cfg['maillist']		= 'none';
// Mail Server (none|sendmail|qmail|virtualqmail)
$cfg['mailserver']	= 'sendmail';
// DNS Server (none|mail|bind|tinydns|mydns|pdns)
$cfg['dns_system']	= 'bind';
// Secondary DNS Server (none|webcp) -- make sure its IP is in 'allow-update'
$cfg['dns2_system'] 	= 'none';
/////////////////////////////////////////////////////////
// 11. Server Configuration
//
// Mailman Config
// In order to use it, you will need to apply a small fix to bin/newlist of Mailman. Place a # in front of line 198.
$cfg['mm_wrap']		= '/home/mailman/mail/wrapper';
$cfg['mm_maxlists']	= 5;
// Webware support -- Use mod_webkit and have "LoadModule libexec/mod_webkit.so" in your primary httpd.conf
$cfg['webware_dir']	= 'webware';
$cfg['webware_init'] 	= '/usr/local/Webware/bin/MakeAppWorkDir.py';
// System log rotate directory (works with cron.daily/logrotate)
$cfg['logdir']		= '/etc/logrotate.d/';
// Server Side HTTP config Location
// Double quotes are used here because single quotes are needed within the strings
$cfg['ss']['perl']	= "\tAddHandler cgi-script .cgi .pl\n\t<Directory '%PATH%'>\n\tOptions +ExecCGI\n\t</Directory>";
$cfg['ss']['php']		= "\tphp_admin_flag engine On\n\tphp_admin_value doc_root %PATH%\n\tphp_admin_value safe_mode_exec_dir .:%PATH%\n\tphp_admin_value open_basedir .:/tmp:".$cfg['php_lib'].":%PATH%\n\tAddType application/x-httpd-php .php .php4 .php3 .phtml\n\tAddType application/x-httpd-php-source .phps";
$cfg['ss']['ssi']		= "\tAddType text/html .shtml\n\tAddHandler server-parsed .shtml\n\t<Directory '%PATH%'>\n\tOptions +Includes\n\t</Directory>";
$cfg['ss']['asp'] 	= "\tAddHandler chiliasp .asp .asa";
$cfg['ss']['cfm'] 	= "\tJRunConfig Verbose False\n\tJRunConfig Apialloc false\n\tJRunConfig Ssl false\n\tJRunConfig Ignoresuffixmap true\n\tJRunConfig Serverstore /usr/coldfusionmx/runtime/lib/wsconfig/1/jrunserver.store\n\tJRunConfig Bootstrap 127.0.0.1:51010\n\tAddHandler jrun-handler .cfm .cfc .jsp .cfml .jws";
$cfg['ss']['webware']   = "\t<Location /".$cfg['webware_dir'].">\n\tWKServer localhost 8086\n\tSetHandler webkit-handler\n\t</Location>";
$cfg['ss']['python'] 	= "";
$cfg['ss']['jsp'] 	= "";
// ProFTPD Configurations
// no settings
// WuFTPd Configurations
// no settings
// Sendmail Configurations
$cfg['mail_spool']	= '/var/spool/mail';
$cfg['mail_access']	= '/etc/mail/access';
$cfg['mail_virtuser']	= '/etc/mail/virtusertable';
$cfg['mail_aliases']	= '/etc/aliases';
$cfg['mail_sendmail']   = '/etc/mail/local-host-names';
// Use UW-IMAP - with black-box mode (true|false)
$cfg['uwimap']		= false;
// UW-IMAP Mailbox Base Directory
$cfg['uwimapdir']		= $cfg['basedir'].'/imap';
// QMail Configurations
$cfg['qmail_rpchosts']		= '/var/qmail/control/rcpthosts'; 
$cfg['qmail_virtdomains']	= '/var/qmail/control/virtualdomains'; 
$cfg['qmail_assign']		= '/var/qmail/users/assign'; 
// Virtual QMail Configurations
$cfg['vqmail_user'] 			= 'vpopmail';
$cfg['vqmail_group'] 			= 'vchkpw';
$cfg['vqmail_domains'] 			= $cfg['vqmail_home'].'/domains';
$cfg['vqmail_postmaster_passwd'] 	= 'CHANGE_PASSWORD';
$cfg['vqmail_rpchosts'] 		= $cfg['qmail_rpchosts'];
$cfg['vqmail_virtdomains'] 		= $cfg['qmail_virtdomains'];
$cfg['vqmail_assign'] 			= $cfg['qmail_assign'];
/////////////////////////////////////////////////////////
// 12. Service Config
//
// Service stop / start / restart
$cfg['init']['timeout']	= 60;
$cfg['init']['httpd']	= '/etc/init.d/httpd';
$cfg['init']['httpd-cp']= '/etc/init.d/httpd-cp';
$cfg['init']['sendmail']= '/etc/init.d/sendmail';
$cfg['init']['pop3']	= '/etc/init.d/sendmail';
$cfg['init']['bind']	= '/etc/init.d/named';
$cfg['init']['qmail']	= '/etc/init.d/qmail';
$cfg['init']['vqmail']	= '/etc/init.d/vpopmail';
$cfg['init']['pureftpd']= '/etc/init.d/pureftpd';
$cfg['init']['proftpd']	= '/etc/init.d/proftpd';
$cfg['init']['mydns']	= '/etc/init.d/mydns';
$cfg['init']['mail']	= '/etc/init.d/sendmail';
// Service PID (for monitoring)
$cfg['spid']['httpd']	= '/var/run/httpd.pid';
$cfg['spid']['httpd-cp']= '/var/run/httpd-cp.pid';
$cfg['spid']['sendmail']= '/var/run/sendmail.out.pid';
$cfg['spid']['qmail']	= '';
$cfg['spid']['bind']	= '/var/run/named/named.pid';
$cfg['spid']['proftpd']	= '/var/run/proftpd.pid';
$cfg['spid']['pureftpd']= '/var/run/pureftpd.pid';
$cfg['spid']['wuftpd']	= '/var/run/wuftpd.pid';
$cfg['spid']['pop3']	= '/var/run/sendmail.in.pid';
$cfg['spid']['imap']	= '/var/run/xinetd.pid';
$cfg['spid']['sshd']	= '/var/run/sshd.pid';
$cfg['spid']['telnet']	= '';
$cfg['spid']['mysql']	= '/var/run/mysqld/mysqld.pid';
$cfg['spid']['pgsql']	= '/var/run/postmaster.pid';
$cfg['spid']['mydns']	= '/var/run/mydns.pid';
$cfg['spid']['mail']	= '/var/run/sendmail.out.pid';
/////////////////////////////////////////////////////////
// 13. DNS Configuration
//
// Directory containing tinydns data file
// ONLY applicable if $cfg["dns_system"] is set to 'tinydns'
$cfg["tinydns_dir"] 	= '/etc/tinydns';
// Default SOA DNS Settings
// Name Servers - #1 is primary (ns1.domain.com|none)
$cfg['dns_server1']	= 'ns0.domain.com';
$cfg['dns_server2']	= 'ns1.domain.com';
$cfg['dns_server3']	= 'none';
$cfg['dns_server4']	= 'none';
// DNS admin e-mail
$cfg['dns_email']		= '';
// BIND configuration
// Allowed zone transfers (from your 'slave' DNS servers) (none|ip address|ip addresses seperated by ;)
$cfg['allow-update']	= 'none';
// Which directory on secondary dns server to use (dnscp) - No ended slash -
$cfg['dns2_dir'] 		= '/etc/named';
// If dns2_system=webcp which file name to use (ns2.named)
// create the file manualy in webcp/web directory
$cfg['dns2_file'] 	= 'ns2.named';
// dns2_system=email Secondary DNS Administrator E-Mail
$cfg['dns2_mail'] 	= $cfg['adminmail'];
// Defaults
// DNS Refresh (10800)
$cfg['dns_refresh']	= 10800;
// DNS Retry (3600)
$cfg['dns_retry']		= 3600;
// DNS Time To Live (86400)
$cfg['dns_ttl']		= 86400;
// DNS minimum TTL (86400)
$cfg['dns_minttl']	= 86400;
// DNS Expire (604800)
$cfg['dns_expire']	= 604800;
// Reverse Records IP Mask
$cfg['dns_ipmask']	= 24;
// Default MyDNS database
$cfg['mydns_db']       = 'mydns';
// Default PowerDNS database
$cfg['pdns_db']         = 'pdns';
/////////////////////////////////////////////////////////
// 14. Mail Exchangers
//
// specify up to four mail exchangers to use with webcp
// IN ORDER OF PREFERENCE:
$cfg['mail_exchanger1'] = 'mx1.example.com';
$cfg['mail_exchanger2'] = 'none';
$cfg['mail_exchanger3'] = 'none';
$cfg['mail_exchanger4'] = 'none';
/////////////////////////////////////////////////////////
// 15. Helpdesk Settings
//
$cfg['helpdesk_mailtype'] = 'pop3/notls';
$cfg['helpdesk_spam'] = '***SPAM***';
$cfg['helpdesk_MI'] = true;
$cfg['helpdesk_POP_host'] = '';
$cfg['helpdesk_POP_port'] = '110';
$cfg['helpdesk_POP_email'] = '';
$cfg['helpdesk_POP_un'] = '';
$cfg['helpdesk_POP_pass'] = 'support';
$cfg['helpdesk_POP_mailbox'] = 'INBOX';
/////////////////////////////////////////////////////////
////////////// CONFIGURATION ENDS HERE //////////////////
/////////////////////////////////////////////////////////
// check for the existence of a custom config.php file
$tmpcfg = explode("/",__FILE__);
array_pop($tmpcfg);array_pop($tmpcfg);
$tmpcfg = implode("/",$tmpcfg)."/config.php";
if (file_exists($tmpcfg))
	include($tmpcfg);
?>
